import fs from 'fs'

import { Request, Response } from 'express'

import { BaseController } from '#application/base/baseController'
import { httpLogger } from '#application/logger'
import { getPublicPath } from '#application/storage'
import { AssetData, UUID } from '#application/types'
import SpriteRepository from '#repositories/spriteRepository'
import TileRepository from '#repositories/tileRepository'
import ZoneRepository from '#repositories/zoneRepository'

export class AssetsController extends BaseController {
  /**
   * List tiles
   * @param req
   * @param res
   */
  public async listTiles(req: Request, res: Response) {
    const assets: AssetData[] = []
    const tiles = await TileRepository.getAll()

    for (const tile of tiles) {
      assets.push({ key: tile.getId(), data: '/assets/tiles/' + tile.getId() + '.png', group: 'tiles', updatedAt: tile.getUpdatedAt() } as AssetData)
    }

    return this.sendSuccess(res, assets)
  }

  /**
   * List tiles by zone
   * @param req
   * @param res
   */
  public async listTilesByZone(req: Request, res: Response) {
    const zoneId = parseInt(req.params.zoneId)

    if (!zoneId || zoneId === 0) {
      return this.sendError(res, 'Invalid zone ID', 400)
    }

    const zone = await ZoneRepository.getById(zoneId)
    if (!zone) {
      return this.sendError(res, 'Zone not found', 404)
    }

    const assets: AssetData[] = []
    const tiles = await TileRepository.getByZoneId(zoneId)

    for (const tile of tiles) {
      assets.push({ key: tile.getId(), data: '/assets/tiles/' + tile.getId() + '.png', group: 'tiles', updatedAt: tile.getUpdatedAt() } as AssetData)
    }

    return this.sendSuccess(res, assets)
  }

  /**
   * List sprite actions
   * @param req
   * @param res
   */
  public async listSpriteActions(req: Request, res: Response) {
    const spriteId = req.params.spriteId as UUID

    if (!spriteId) {
      return this.sendError(res, 'Invalid sprite ID', 400)
    }

    const sprite = await SpriteRepository.getById(spriteId)
    if (!sprite) {
      return this.sendError(res, 'Sprite not found', 404)
    }

    const assets: AssetData[] = sprite.spriteActions.getItems().map((spriteAction) => ({
      key: sprite.id + '-' + spriteAction.action,
      data: '/assets/sprites/' + sprite.getId() + '/' + spriteAction.getAction() + '.png',
      group: spriteAction.isAnimated ? 'sprite_animations' : 'sprites',
      updatedAt: sprite.getUpdatedAt(),
      originX: Number(spriteAction.originX.toString()),
      originY: Number(spriteAction.originY.toString()),
      isAnimated: spriteAction.getIsAnimated(),
      frameRate: spriteAction.getFrameRate(),
      frameWidth: spriteAction.getFrameWidth(),
      frameHeight: spriteAction.getFrameHeight(),
      frameCount: JSON.parse(JSON.stringify(spriteAction.getSprites())).length
    }))

    return this.sendSuccess(res, assets)
  }

  /**
   * Download asset
   * @param req
   * @param res
   */
  public async downloadAsset(req: Request, res: Response) {
    const { type, spriteId, file } = req.params

    const assetPath = type === 'sprites' && spriteId ? getPublicPath(type, spriteId, file) : getPublicPath(type, file)

    if (!fs.existsSync(assetPath)) {
      httpLogger.error(`File not found: ${assetPath}`)
      return this.sendError(res, 'Asset not found', 404)
    }

    res.sendFile(assetPath, (err) => {
      if (err) {
        httpLogger.error('Error sending file:', err)
        this.sendError(res, 'Error downloading the asset', 500)
      }
    })
  }
}
